﻿using DevConf22.Web.Models;
using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;
using Dapr.Client;

namespace DevConf22.Web.Controllers;
public class HomeController : Controller
{
    private readonly ILogger<HomeController> _logger;

    public HomeController(ILogger<HomeController> logger)
    {
        _logger = logger;
    }

    public async Task<IActionResult> Index(
        [FromServices]DaprClient client)
    {
        var orders = await client.GetStateAsync<string[]>("statestore", "orders")
                     ?? Array.Empty<string>();
        return View(orders);
    }

    [ActionName("index")]
    [HttpPost]
    public async Task<IActionResult> PostIndex(
        [FromServices] DaprClient client)
    {
        await client.PublishEventAsync("pubsub", "orders", DateTimeOffset.Now);

        return RedirectToAction();
    }

    public IActionResult Privacy()
    {
        return View();
    }

    [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
    public IActionResult Error()
    {
        return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
    }
}
