using System.Text.Json.Serialization;
using Dapr.Client;

var builder = WebApplication.CreateBuilder(args);

builder.Services.AddDaprClient();

var app = builder.Build();

if (app.Environment.IsDevelopment()) { app.UseDeveloperExceptionPage(); }

app.MapGet("/dapr/subscribe", (ILogger<Program> logger) => {
    var sub = new DaprSubscription(PubsubName: "pubsub", Topic: "orders", Route: "orders");

    logger.LogInformation("Dapr pub/sub is subscribed to {sub}", sub);
    return Results.Json(new [] { sub });
});

app.MapPost("/orders", async (DaprData<DateTimeOffset> requestData, DaprClient client, ILogger<Program> logger) => {
    logger.LogInformation("Received order on {date}", requestData.Data);

    // Leggo lo stato
    var orders = (
        await client.GetStateAsync<string[]>("statestore", "orders")
                 ?? Array.Empty<string>()).ToList();

    logger.LogInformation("items {count}", orders.Count);
    orders.Add($"Processed order {requestData.Data}");

    // Salvo lo stato
    await client.SaveStateAsync("statestore", "orders", orders.ToArray());

    return Results.Ok(requestData.Data);
});

await app.RunAsync();

public record DaprData<T>([property: JsonPropertyName("data")] T Data);
public record DaprSubscription(
    [property: JsonPropertyName("pubsubname")] string PubsubName,
    [property: JsonPropertyName("topic")] string Topic,
    [property: JsonPropertyName("route")] string Route);